# Generated by Django 4.2.23 on 2025-07-04 23:56

import django.contrib.auth.models
import django.contrib.auth.validators
from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='API',
            fields=[
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('method', models.CharField(default='GET', max_length=18, verbose_name='Method')),
                ('api', models.CharField(max_length=255, verbose_name='API API')),
                ('func_name', models.CharField(max_length=255, verbose_name='API Name')),
                ('name', models.CharField(default='', max_length=255, verbose_name='API Description')),
            ],
            options={
                'verbose_name': 'bomiot API',
                'verbose_name_plural': 'bomiot API',
                'db_table': 'bomiot_api',
                'ordering': ['id'],
            },
        ),
        migrations.CreateModel(
            name='ASN',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('data', models.JSONField()),
            ],
            options={
                'verbose_name': 'bomiot ASN',
                'verbose_name_plural': 'bomiot ASN',
                'db_table': 'bomiot_asn',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Bar',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('data', models.JSONField()),
            ],
            options={
                'verbose_name': 'bomiot Bar',
                'verbose_name_plural': 'bomiot Bar',
                'db_table': 'bomiot_bar',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Bin',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('data', models.JSONField()),
            ],
            options={
                'verbose_name': 'bomiot Bin',
                'verbose_name_plural': 'bomiot Bin',
                'db_table': 'bomiot_bin',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Capital',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('data', models.JSONField()),
            ],
            options={
                'verbose_name': 'bomiot Capital',
                'verbose_name_plural': 'bomiot Capital',
                'db_table': 'bomiot_capital',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='CPU',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('cpu_usage', models.FloatField(default=0, verbose_name='CPU Usage')),
                ('physical_cores', models.IntegerField(default=0, verbose_name='Physical Cores')),
                ('logical_cores', models.IntegerField(default=0, verbose_name='Logical Cores')),
                ('cpu_frequency', models.CharField(default='', max_length=255, verbose_name='CPU Frequency')),
                ('min_cpu_frequency', models.CharField(default='', max_length=255, verbose_name='Min CPU Frequency')),
                ('max_cpu_frequency', models.CharField(default='', max_length=255, verbose_name='Max CPU Frequency')),
            ],
            options={
                'verbose_name': 'bomiot CPU',
                'verbose_name_plural': 'bomiot CPU',
                'db_table': 'bomiot_cpu',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Customer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('data', models.JSONField()),
            ],
            options={
                'verbose_name': 'bomiot Customer',
                'verbose_name_plural': 'bomiot Customer',
                'db_table': 'bomiot_customer',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Department',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('name', models.CharField(default='', max_length=255, verbose_name='Department Name')),
            ],
            options={
                'verbose_name': 'bomiot Department',
                'verbose_name_plural': 'bomiot Department',
                'db_table': 'bomiot_department',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Disk',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('device', models.CharField(default='', max_length=255, verbose_name='Device')),
                ('mountpoint', models.CharField(default='', max_length=255, verbose_name='Mountpoint')),
                ('total', models.BigIntegerField(default=0, verbose_name='Total')),
                ('used', models.BigIntegerField(default=0, verbose_name='Used')),
                ('free', models.BigIntegerField(default=0, verbose_name='Free')),
                ('percent', models.FloatField(default=0, verbose_name='Percent')),
            ],
            options={
                'verbose_name': 'bomiot Disk',
                'verbose_name_plural': 'bomiot Disk',
                'db_table': 'bomiot_disk',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='DN',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('data', models.JSONField()),
            ],
            options={
                'verbose_name': 'bomiot DN',
                'verbose_name_plural': 'bomiot DN',
                'db_table': 'bomiot_dn',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Driver',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('data', models.JSONField()),
            ],
            options={
                'verbose_name': 'bomiot Driver',
                'verbose_name_plural': 'bomiot Driver',
                'db_table': 'bomiot_driver',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Example',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('data', models.JSONField()),
            ],
            options={
                'verbose_name': 'bomiot Example',
                'verbose_name_plural': 'bomiot Example',
                'db_table': 'bomiot_example',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Fee',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('data', models.JSONField()),
            ],
            options={
                'verbose_name': 'bomiot Fee',
                'verbose_name_plural': 'bomiot Fee',
                'db_table': 'bomiot_fee',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Files',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('name', models.CharField(blank=True, default='', max_length=255, verbose_name='File Name')),
                ('type', models.CharField(blank=True, default='', max_length=255, verbose_name='Type')),
                ('size', models.BigIntegerField(blank=True, default=0, verbose_name='Size')),
                ('owner', models.CharField(blank=True, default='', max_length=255, verbose_name='Owner')),
                ('shared_to', models.CharField(blank=True, default='', max_length=255, verbose_name='Shared To')),
            ],
            options={
                'verbose_name': 'bomiot Files',
                'verbose_name_plural': 'bomiot Files',
                'db_table': 'bomiot_files',
                'ordering': ['-updated_time'],
            },
        ),
        migrations.CreateModel(
            name='Goods',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('data', models.JSONField()),
            ],
            options={
                'verbose_name': 'bomiot Goods',
                'verbose_name_plural': 'bomiot Goods',
                'db_table': 'bomiot_goods',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='JobList',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('job_id', models.CharField(max_length=255, verbose_name='Job ID')),
                ('module_name', models.CharField(max_length=255, verbose_name='Module Name')),
                ('func_name', models.CharField(max_length=255, verbose_name='Function Name')),
                ('trigger', models.CharField(max_length=255, verbose_name='Trigger')),
                ('description', models.TextField(blank=True, default='', null=True, verbose_name='Description')),
                ('configuration', models.TextField(null=True, verbose_name='Configuration')),
                ('type', models.BooleanField(default=True, verbose_name='Type')),
            ],
            options={
                'verbose_name': 'bomiot Job',
                'verbose_name_plural': 'bomiot Job',
                'db_table': 'bomiot_job',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Memory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('total', models.BigIntegerField(default=0, verbose_name='Total Memory')),
                ('used', models.BigIntegerField(default=0, verbose_name='Used Memory')),
                ('free', models.BigIntegerField(default=0, verbose_name='Free Memory')),
                ('percent', models.FloatField(default=0, verbose_name='Percent')),
                ('swap_total', models.BigIntegerField(default=0, verbose_name='Swap Total Memory')),
                ('swap_used', models.BigIntegerField(default=0, verbose_name='Swap Used Memory')),
                ('swap_free', models.BigIntegerField(default=0, verbose_name='Swap Free Memory')),
                ('swap_percent', models.FloatField(default=0, verbose_name='Swap Percent')),
            ],
            options={
                'verbose_name': 'bomiot Memory',
                'verbose_name_plural': 'bomiot Memory',
                'db_table': 'bomiot_memory',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('sender', models.CharField(blank=True, default='', max_length=255, verbose_name='Sender')),
                ('receiver', models.CharField(blank=True, default='', max_length=255, verbose_name='Receiver')),
                ('detail', models.CharField(default='', max_length=255, verbose_name='Detail')),
                ('can_send', models.BooleanField(default=False, verbose_name='Can Send')),
            ],
            options={
                'verbose_name': 'bomiot Message',
                'verbose_name_plural': 'bomiot Message',
                'db_table': 'bomiot_message',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Network',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('bytes_sent', models.BigIntegerField(default=0, verbose_name='Bytes Sent')),
                ('bytes_recv', models.BigIntegerField(default=0, verbose_name='Bytes Received')),
            ],
            options={
                'verbose_name': 'bomiot Network',
                'verbose_name_plural': 'bomiot Network',
                'db_table': 'bomiot_network',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Permission',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('api', models.CharField(max_length=255, verbose_name='Permission API')),
                ('name', models.CharField(max_length=255, verbose_name='Permission Name')),
            ],
            options={
                'verbose_name': 'bomiot Permission',
                'verbose_name_plural': 'bomiot Permission',
                'db_table': 'bomiot_permission',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Pids',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('pid', models.IntegerField(default=0, verbose_name='PID')),
                ('name', models.CharField(blank=True, default='', max_length=255, verbose_name='PID Name')),
                ('memory', models.BigIntegerField(default=0, verbose_name='Memory')),
                ('create_time', models.DateTimeField(blank=True, verbose_name='Create Time')),
                ('memory_usage', models.FloatField(default=0, verbose_name='Memory Usage')),
                ('cpu_usage', models.FloatField(default=0, verbose_name='CPU Usage')),
            ],
            options={
                'verbose_name': 'bomiot Pids',
                'verbose_name_plural': 'bomiot Pids',
                'db_table': 'bomiot_pids',
                'ordering': ['-cpu_usage'],
            },
        ),
        migrations.CreateModel(
            name='Purchase',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('data', models.JSONField()),
            ],
            options={
                'verbose_name': 'bomiot Purchase',
                'verbose_name_plural': 'bomiot Purchase',
                'db_table': 'bomiot_purchase',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='PyPi',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('category', models.CharField(max_length=255, verbose_name='Category')),
                ('date', models.DateTimeField(blank=True, verbose_name='PyPi Date')),
                ('percent', models.FloatField(default=0, verbose_name='Percent')),
                ('downloads', models.IntegerField(default=0, verbose_name='Downloads')),
            ],
            options={
                'verbose_name': 'bomiot PyPi',
                'verbose_name_plural': 'bomiot PyPi',
                'db_table': 'bomiot_pypi',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Stock',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('data', models.JSONField()),
            ],
            options={
                'verbose_name': 'bomiot Stock',
                'verbose_name_plural': 'bomiot Stock',
                'db_table': 'bomiot_stock',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Supplier',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('data', models.JSONField()),
            ],
            options={
                'verbose_name': 'bomiot Supplier',
                'verbose_name_plural': 'bomiot Supplier',
                'db_table': 'bomiot_supplier',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Team',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('name', models.CharField(default='', max_length=255, verbose_name='Team Name')),
                ('permission', models.JSONField(default=dict, null=True, verbose_name='Permission')),
            ],
            options={
                'verbose_name': 'bomiot Team',
                'verbose_name_plural': 'bomiot Team',
                'db_table': 'bomiot_team',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='ThrottleModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('ip', models.CharField(max_length=255, verbose_name='IP')),
                ('method', models.CharField(max_length=18, verbose_name='Method')),
            ],
            options={
                'verbose_name': 'bomiot Throttle',
                'verbose_name_plural': 'bomiot Throttle',
                'db_table': 'bomiot_throttle',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('username', models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='username')),
                ('first_name', models.CharField(blank=True, max_length=150, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=150, verbose_name='last name')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='email address')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('is_delete', models.BooleanField(default=False, verbose_name='Delete Label')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='Created Time')),
                ('updated_time', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated Time')),
                ('type', models.IntegerField(default=1, verbose_name='User Type')),
                ('phone', models.CharField(blank=True, default='', max_length=255, verbose_name='Phone')),
                ('permission', models.JSONField(default=dict, null=True, verbose_name='Permission')),
                ('request_limit', models.IntegerField(default=0, verbose_name='Request Limit')),
                ('team', models.IntegerField(blank=True, default=0, verbose_name='Team')),
                ('department', models.IntegerField(blank=True, default=0, verbose_name='Department')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'bomiot User',
                'verbose_name_plural': 'bomiot User',
                'db_table': 'bomiot_user',
                'ordering': ['-id'],
                'abstract': False,
            },
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
    ]



import os
import re

from os.path import join, exists
from rest_framework import viewsets
from rest_framework.response import Response
from rest_framework.exceptions import APIException
from rest_framework.parsers import MultiPartParser, FormParser
from functools import reduce

from bomiot.server.core import serializers, models, filter
from bomiot.server.core.page import CorePageNumberPagination, PermissionPageNumberPagination, APIPageNumberPagination, TeamPageNumberPagination
from bomiot.server.core.permission import NormalPermission
from bomiot.server.core.utils import readable_file_size, sync_write_file
from bomiot.server.core.signal import bomiot_signals
from rest_framework.filters import OrderingFilter
from rest_framework.exceptions import MethodNotAllowed
from django_filters.rest_framework import DjangoFilterBackend
from bomiot_message import permission_message_return, detail_message_return, msg_message_return, others_message_return

from django.contrib.auth import get_user_model
from django.db.models import Q
from django.conf import settings

User = get_user_model()


class UserPage(CorePageNumberPagination):

    def get_return_data(self, data) -> dict:
        return {"label": data.name, "value": data.id}

    def query_data_add(self) -> list:
        team_list = models.Team.objects.all()
        team_data_list = list(map(lambda data: self.get_return_data(data), team_list))
        department_list = models.Department.objects.all()
        department_data_list = list(map(lambda data: self.get_return_data(data), department_list))
        return [
            ("team", team_data_list),
            ("department", department_data_list)
        ]


class UserList(viewsets.ModelViewSet):
    """
        list:
            Response a user data list（all）
    """
    pagination_class = UserPage
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "username", "created_time", "updated_time", ]
    filter_class = filter.UserFilter

    def get_queryset(self):
        if self.request.user:
            query_data = {'is_delete': False}
            search = self.request.query_params.get('search')
            if search:
                query_data['username__icontains'] = search
            return User.objects.filter(**query_data).order_by('-date_joined')
        else:
            return User.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.UserSerializer
        else:
            raise MethodNotAllowed(self.request.method)
            
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)

class PermissionList(viewsets.ModelViewSet):
    """
        list:
            Response a permission data list（all）
    """
    pagination_class = PermissionPageNumberPagination
    authentication_classes = []
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.PermissionFilter

    def get_queryset(self):
        if self.request.user:
            return models.Permission.objects.filter().order_by('-id')
        else:
            return models.Permission.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.PermissionSerializer
        else:
            raise MethodNotAllowed(self.request.method)
        
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)


class UserCreate(viewsets.ModelViewSet):
    """
        create:
            create a user
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.UserFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.UserSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, **kwargs):
        data = self.request.data
        user_check = User.objects.filter(username=data.get('username'), is_delete=False)
        if user_check.exists():
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "User exists"))
        else:
            if self.request.auth.is_superuser is True:
                User.objects.create_user(username=data.get('username'), password=data.get('username'))
                user_folder = join(settings.MEDIA_ROOT, data.get('username'))
                exists(user_folder) or os.makedirs(user_folder)
            else:
                if "Create One User" not in self.request.auth.permission:
                    raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                             "User does not have permission to create user"))
                else:
                    User.objects.create_user(username=data.get('username'), password=data.get('username'))
                    user_folder = join(settings.MEDIA_ROOT, data.get('username'))
                    exists(user_folder) or os.makedirs(user_folder)
        return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                           "Success Create User"), status=200)


class UserChangePWD(viewsets.ModelViewSet):
    """
        create:
            Change Password
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.UserFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.UserSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, **kwargs):
        data = self.request.data
        user_check = User.objects.filter(id=data.get('id'), is_delete=False)
        if user_check.exists() is False:
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "User not exists"))
        else:
            user_data = user_check.first()
            if self.request.auth.is_superuser is True:
                user_data.set_password(str(data.get('pwd')))
                user_data.save()
            else:
                if "Change Password" not in self.request.auth.permission:
                    raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                             "User does not have permission to change password"))
                else:
                    if self.request.auth.id == int(data.get('id')):
                        user_data.set_password(str(data.get('pwd')))
                        user_data.save()
                    else:
                        raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                                 "User can only change your own password"))
            return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                               "Success Change Password"), status=200)


class UserSetTeam(viewsets.ModelViewSet):
    """
        create:
            set team for user
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.UserFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.UserSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, **kwargs):
        data = self.request.data
        user_check = User.objects.filter(id=int(data.get('id')), is_delete=False)
        if user_check.exists() is False:
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "User not exists"))
        else:
            user_data = user_check.first()
            if "Set Team For User" not in self.request.auth.permission:
                raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                         "User does not have permission to set team for user"))
            else:
                team_data = models.Team.objects.filter(id=int(data.get('team_id')), is_delete=False).first()
                user_data.permission = team_data.permission if team_data else {}
                user_data.team = int(data.get('team_id'))
                user_data.save()
        return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                            "Success set team"), status=200)


class UserSetDepartment(viewsets.ModelViewSet):
    """
        create:
            set department for user
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.UserFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.UserSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, **kwargs):
        data = self.request.data
        user_check = User.objects.filter(id=data.get('id'), is_delete=False)
        if user_check.exists() is False:
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "User not exists"))
        else:
            user_data = user_check.first()
            if "Set Department For User" not in self.request.auth.permission:
                raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                         "User does not have permission to set department for user"))
            else:
                user_data.department = data.get('department_id')
                user_data.save()
        return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                            "Success set department"), status=200)


class UserLock(viewsets.ModelViewSet):
    """
        create:
            Lock one User
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.UserFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.UserSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, **kwargs):
        data = self.request.data
        user_check = User.objects.filter(id=data.get('id'), is_delete=False)
        if user_check.exists() is False:
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "User not exists"))
        else:
            user_data = user_check.first()
            if user_data.is_superuser:
                raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                         "Can not lock admin"))
            else:
                if "Lock & Unlock User" not in self.request.auth.permission:
                    raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                             "User does not have permission to lock user"))
                else:
                    if self.request.auth.id == int(data.get('id')):
                        raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                                 "User can not lock/unlock your own"))
                    else:
                        if user_data.is_active is True:
                            user_data.is_active = False
                            user_data.request_limit = 0
                            user_data.save()
                            return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                               "Success lock User"), status=200)
                        else:
                            user_data.is_active = True
                            user_data.request_limit = 0
                            user_data.save()
                            return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                               "Success unlock User"), status=200)


class UserDelete(viewsets.ModelViewSet):
    """
        create:
            Delete one User
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.UserFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.UserSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, **kwargs):
        data = self.request.data
        user_check = User.objects.filter(id=data.get('id'), is_delete=False)
        if user_check.exists() is False:
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "User not exists"))
        else:
            user_data = user_check.first()
            if user_data.is_superuser:
                raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                         "Can not delete admin"))
            else:
                if "Delete One User" not in self.request.auth.permission:
                    raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                             "User does not have permission to delete user"))
                else:
                    if self.request.auth.id == int(data.get('id')):
                        raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                                 "User can not delete your own"))
                    else:
                        user_data.is_delete = True
                        return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                           "Success delete User"), status=200)


class UserUpload(viewsets.ModelViewSet):
    """
        create:
            upload files
    """
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.FileFilter
    parser_classes = (MultiPartParser, FormParser)

    def create(self, request, *args, **kwargs):
        data = self.request.FILES
        exists(join(settings.MEDIA_ROOT, self.request.auth.username)) or os.makedirs(join(settings.MEDIA_ROOT, self.request.auth.username))
        for file_name in data:
            file_obj = self.request.FILES.get(file_name)
            file_extension = file_obj.name.split('.')[-1].lower()
            if file_extension not in settings.FILE_EXTENSION:
                detail = others_message_return(self.request.META.get('HTTP_LANGUAGE', ''), "File type not allowed")
                context = {'detail': f"{detail} {file_obj.name}"}
                raise APIException(context)
            if int(file_obj.size) >= int(settings.FILE_SIZE):
                file_size = readable_file_size(file_obj.size)
                file_size_limit = readable_file_size(settings.FILE_SIZE)
                detail = others_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                         "File size exceeds limit")
                context = {"detail": f"{detail} {file_size}/{file_size_limit}"}
                raise APIException(context)
            file_data = file_obj.read()
            file_path = join(settings.MEDIA_ROOT, str(self.request.auth.username), file_obj.name)
            bomiot_signals.send(
                sender=sync_write_file,
                msg={'models': 'Function'},
                file_path=file_path,
                file_data=file_data
            )
            context = {}
            msg = others_message_return(self.request.META.get('HTTP_LANGUAGE', ''), "Success upload files")
            context = {"msg": f"{msg} {file_obj.name}"}
            return Response(context, status=200)
                


class FilePage(CorePageNumberPagination):

    def get_user_return_data(self, data):
        return {"label": data.username, "value": data.id}

    def query_data_add(self) -> list:
        user_list = User.objects.filter(is_delete=False).exclude(id=self.request.auth.id)
        user_data_list = list(map(lambda data: self.get_user_return_data(data), user_list))

        return [
            ("users", user_data_list)
        ]


class UserFiles(viewsets.ModelViewSet):
    """
        list:
            Response a user file list（all）
    """
    pagination_class = FilePage
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.FileFilter

    def get_queryset(self):
        if self.request.user:
            query_data = {'is_delete': False}
            search = self.request.query_params.get('search')
            if search:
                query_data['name__icontains'] = search
            if self.request.auth.is_superuser is True:
                return models.Files.objects.filter(**query_data).order_by('-updated_time')
            else:
                share_user = '<->' + str(self.request.auth.id) + "<->" + self.request.auth.username
                return models.Files.objects.filter(
                                            Q(owner=self.request.auth.username, **query_data)
                                            |
                                            Q(shared_to__icontains=share_user, **query_data)
                                            ).order_by('-updated_time')
        else:
            return models.Files.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.FileSerializer
        else:
            raise MethodNotAllowed(self.request.method)
                
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)


class FileShare(viewsets.ModelViewSet):
    """
        create:
            Share your file
    """
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.FileFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.FileSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def get_users_data(self, data):
        user_data = User.objects.filter(id=data).first()
        return "<->" + str(data) + "<->" + user_data.username

    def create(self, request, **kwargs):
        data = self.request.data
        file_check = models.Files.objects.filter(id=data.get('id'), is_delete=False)
        if file_check.exists() is False:
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "File not exists"))
        else:
            file_data = file_check.first()
            if file_data.owner != self.request.auth.username:
                raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                         "Can not share file which is not your own"))
            else:
                users_list = data.get('users')
                data_list = list(map(lambda data: self.get_users_data(data), users_list))
                shared_to_data = ",".join(data_list)
                file_data.shared_to = shared_to_data
                file_data.save()
            return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                               "Success share your file"), status=200)


class DeleteFile(viewsets.ModelViewSet):
    """
        create:
            Delete one file
    """
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.FileFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.FileSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, **kwargs):
        data = self.request.data
        file_check = models.Files.objects.filter(id=data.get('id'), is_delete=False)
        if file_check.exists() is False:
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "File not exists"))
        else:
            file_data = file_check.first()
            if self.request.auth.is_superuser is True:
                file_data.is_delete = True
            else:
                if file_data.owner != self.request.auth.username:
                    raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                             "Can not delete file which is not your own"))
                else:
                    file_data.is_delete = True
            file_data.save()
            return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                               "Success delete file"), status=200)


class TeamList(viewsets.ModelViewSet):
    """
        list:
            Response a team data list（all）
    """
    pagination_class = TeamPageNumberPagination
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "name", "created_time", "updated_time", ]
    filter_class = filter.TeamFilter

    def get_queryset(self):
        if self.request.user:
            query_data = {'is_delete': False}
            search = self.request.query_params.get('search')
            if search:
                query_data['name__icontains'] = search
            return models.Team.objects.filter(**query_data).order_by('-id')
        else:
            return models.Team.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.TeamSerializer
        else:
            raise MethodNotAllowed(self.request.method)
            
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)


class TeamCreate(viewsets.ModelViewSet):
    """
        create:
            create a team
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.TeamFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.TeamSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, **kwargs):
        data = self.request.data
        team_check = models.Team.objects.filter(name=data.get('name'), is_delete=False)
        if team_check.exists():
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "Team exists"))
        else:
            if self.request.auth.is_superuser is True:
                models.Team.objects.create(name=data.get('name'))
            else:
                if "Create One Team" not in self.request.auth.permission:
                    raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                             "User does not have permission to create team"))
                else:
                    models.Team.objects.create(name=data.get('name'))
        return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                           "Success create team"), status=200)


class TeamPermission(viewsets.ModelViewSet):
    """
        create:
            Set permission for team
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.TeamFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.TeamSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def get_permission_data(self, data):
        permission_data = models.Permission.objects.filter(name=data).first()
        return {
            permission_data.name: permission_data.api
        }

    def create(self, request, **kwargs):
        data = self.request.data
        team_check = models.Team.objects.filter(id=data.get('id'), is_delete=False)
        if team_check.exists() is False:
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "Team not exists"))
        else:
            team_data = team_check.first()
            if "Set Permission For Team" not in self.request.auth.permission:
                raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                         "User does not have permission to set permission for team"))
            else:
                permission_list = data.get('permission')
                data_list = list(map(lambda data: self.get_permission_data(data), permission_list))
                permission_data = reduce(lambda x, y: {**x, **y}, data_list)
                team_data.permission = permission_data
                User.objects.filter(team=team_data.id, is_delete=False).update(
                    permission=permission_data
                )
                team_data.save()
        return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                            "Success change team permission"), status=200)


class TeamChange(viewsets.ModelViewSet):
    """
        create:
            Change one team
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.TeamFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.TeamSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, **kwargs):
        data = self.request.data
        team_check = models.Team.objects.filter(name=data.get('name'), is_delete=False)
        if team_check.exists():
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "Team exists"))
        else:
            if self.request.auth.is_superuser is True:
                models.Team.objects.filter(id=data.get('id')).update(name=data.get('name'))
            else:
                if "Change Team" not in self.request.auth.permission:
                    raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                             "User does not have permission to change team"))
                else:
                    models.Team.objects.filter(id=data.get('id')).update(name=data.get('name'))
        return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                           "Success change team"), status=200)


class TeamDelete(viewsets.ModelViewSet):
    """
        create:
            Delete one team
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.TeamFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.TeamSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, **kwargs):
        data = self.request.data
        team_check = models.Team.objects.filter(id=data.get('id'), is_delete=False)
        if team_check.exists() is False:
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "Team not exists"))
        else:
            if self.request.auth.is_superuser is True:
                models.Team.objects.filter(id=data.get('id')).update(is_delete=True)
            else:
                if "Delete Team" not in self.request.auth.permission:
                    raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                             "User does not have permission to delete team"))
                else:
                    models.Team.objects.filter(id=data.get('id')).update(is_delete=True)
        return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                           "Success delete team"), status=200)


class DepartmentList(viewsets.ModelViewSet):
    """
        list:
            Response a department data list（all）
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "name", "created_time", "updated_time", ]
    filter_class = filter.DepartmentFilter

    def get_queryset(self):
        if self.request.user:
            query_data = {'is_delete': False}
            search = self.request.query_params.get('search')
            if search:
                query_data['name__icontains'] = search
            return models.Department.objects.filter(**query_data).order_by('-id')
        else:
            return models.Department.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.DepartmentSerializer
        else:
            raise MethodNotAllowed(self.request.method)
            
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)


class DepartmentCreate(viewsets.ModelViewSet):
    """
        create:
            create one department
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.DepartmentFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.DepartmentSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, **kwargs):
        data = self.request.data
        department_check = models.Department.objects.filter(name=data.get('name'), is_delete=False)
        if department_check.exists():
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "Department exists"))
        else:
            if self.request.auth.is_superuser is True:
                models.Department.objects.create(name=data.get('name'))
            else:
                if "Create department" not in self.request.auth.permission:
                    raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                             "User does not have permission to create department"))
                else:
                    models.Department.objects.create(name=data.get('name'))
        return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                           "Success create department"), status=200)


class DepartmentChange(viewsets.ModelViewSet):
    """
        create:
            Change one department
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.DepartmentFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.DepartmentSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, **kwargs):
        data = self.request.data
        department_check = models.Department.objects.filter(name=data.get('name'), is_delete=False)
        if department_check.exists():
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "Department exists"))
        else:
            if self.request.auth.is_superuser is True:
                models.Department.objects.filter(id=data.get('id')).update(name=data.get('name'))
            else:
                if "Change department" not in self.request.auth.permission:
                    raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                             "User does not have permission to change department"))
                else:
                    models.Department.objects.filter(id=data.get('id')).update(name=data.get('name'))
        return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                           "Success change department"), status=200)


class DepartmentDelete(viewsets.ModelViewSet):
    """
        create:
            Delete one department
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.DepartmentFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.DepartmentSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, **kwargs):
        data = self.request.data
        department_check = models.Department.objects.filter(id=data.get('id'), is_delete=False)
        if department_check.exists() is False:
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "Department not exists"))
        else:
            if self.request.auth.is_superuser is True:
                models.Department.objects.filter(id=data.get('id')).update(is_delete=True)
            else:
                if "Delete department" not in self.request.auth.permission:
                    raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                             "User does not have permission to delete department"))
                else:
                    models.Department.objects.filter(id=data.get('id')).update(is_delete=True)
        return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                           "Success delete department"), status=200)


class APIList(viewsets.ModelViewSet):
    """
        list:
            Response a api data list（all）
    """
    pagination_class = APIPageNumberPagination
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "api", "func_name", "created_time", "updated_time", ]
    filter_class = filter.APIFilter

    def get_queryset(self):
        if self.request.user:
            query_data = {'is_delete': False}
            search = self.request.query_params.get('search')
            if search:
                query_data['api__icontains'] = search
            return models.API.objects.filter(**query_data).order_by('id')
        else:
            return models.API.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.APISerializer
        else:
            raise MethodNotAllowed(self.request.method)
            
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)


class APIChange(viewsets.ModelViewSet):
    """
        create:
            Change one API
    """
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.APIFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.APISerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, **kwargs):
        data = self.request.data
        data_check = bool(re.search(r'\s', data))
        if data_check is True:
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "API name can not contain space"))
        department_check = models.API.objects.filter(func_name=data.get('func_name'), is_delete=False)
        if department_check.exists():
            raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                     "API exists"))
        else:
            
            if self.request.auth.is_superuser is True:
                models.API.objects.filter(id=data.get('id')).update(func_name=data.get('func_name'))
            else:
                raise APIException(detail_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                                             "User does not have permission to change API"))
        return Response(msg_message_return(self.request.META.get('HTTP_LANGUAGE', ''),
                                           "Success change API"), status=200)


class PyPiList(viewsets.ModelViewSet):
    """
        list:
            Response a PyPi data list（all）
    """
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ["id", "created_time", "updated_time", ]
    filter_class = filter.PyPiFilter

    def get_queryset(self):
        if self.request.user:
            query_data = {}
            query_data['is_delete'] = False
            query_data['category__icontains'] = self.request.query_params.get('search')
            return models.PyPi.objects.filter(**query_data).order_by('-date')
        else:
            return models.PyPi.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.PyPiSerializer
        else:
            raise MethodNotAllowed(self.request.method)
        
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)


class PyPiChartsPage(CorePageNumberPagination):
    def __init__(self, *args, **kwargs):
        self.pypi_data = {}
        self.pypi_data['title'] = 'Bomiot PyPi Downloads Pie Chart'
        self.pypi_data['xAxis_list'] = []
        self.with_mirrors_list = []
        self.without_mirrors_list = []
        super(PyPiChartsPage, self).__init__(*args, **kwargs)
    
    def get_pypi_return_data(self, data) -> dict:
        date_check = data.date.strftime("%m-%d")
        if date_check in self.pypi_data['xAxis_list']:
            return data
        else:
            self.pypi_data['xAxis_list'].append(date_check)
            with_mirrors_check =  models.PyPi.objects.filter(category='with_mirrors', date=data.date)
            without_mirrors_check =  models.PyPi.objects.filter(category='without_mirrors', date=data.date)
            if with_mirrors_check.exists():
                self.with_mirrors_list.append(with_mirrors_check.first().downloads)
            else:
                self.with_mirrors_list.append(0)
            if without_mirrors_check.exists():
                self.without_mirrors_list.append(without_mirrors_check.first().downloads)
            else:
                self.without_mirrors_list.append(0)
        return data

    def query_data_add(self) -> list:
        pypi_list = models.PyPi.objects.filter().order_by('-date')[:90]
        pypi_data_list = list(map(lambda data: self.get_pypi_return_data(data), pypi_list))

        self.pypi_data['xAxis_list'] = list(reversed(self.pypi_data['xAxis_list']))
        self.pypi_data['series_list'] = [
            { 'name': "with_mirrors", 'data': list(reversed(self.with_mirrors_list)) },
            { 'name': "without_mirrors", 'data': list(reversed(self.without_mirrors_list)) }
        ]

        return [
            ("pypi_data", self.pypi_data),
        ]


class PyPiCharts(viewsets.ModelViewSet):
    """
        list:
            Response a PID Charts data list（all）
    """
    pagination_class = PyPiChartsPage
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ["id", "created_time", "updated_time", ]
    filter_class = filter.PyPiFilter

    def get_queryset(self):
        return models.PyPi.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.PyPiSerializer
        else:
            raise MethodNotAllowed(self.request.method)
        
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)




from pathlib import Path
from threading import Thread, Timer
from django.conf import settings
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler, FileSystemEvent
from watchdog.utils.dirsnapshot import DirectorySnapshot, DirectorySnapshotDiff
from bomiot.server.core.models import Files
from bomiot.server.core.utils import readable_file_size
from django.contrib.auth import get_user_model
from django.forms import model_to_dict
from bomiot.server.core.utils import compare_dicts
from bomiot.server.core.signal import bomiot_signals

User = get_user_model()


class MyHandler(FileSystemEventHandler):
    def __init__(self):
        FileSystemEventHandler.__init__(self)
        self.timer = None
        self.snapshot = DirectorySnapshot(settings.MEDIA_ROOT)

    def on_any_event(self, event):
        if self.timer:
            self.timer.cancel()
        self.timer = Timer(0.2, self.check_snapshot)
        self.timer.start()

    def check_snapshot(self):
        snapshot = DirectorySnapshot(settings.MEDIA_ROOT)
        diff = DirectorySnapshotDiff(self.snapshot, snapshot)
        self.snapshot = snapshot
        self.timer = None

        created_file_list = list(map(lambda data: self.created_file(data), diff.files_created))
        modified_file_list = list(map(lambda data: self.modified_file(data), diff.files_modified))
        deleted_file_list = list(map(lambda data: self.deleted_file(data), diff.files_deleted))
        moved_file_list = list(map(lambda data: self.moved_file(data), diff.files_moved))

    def created_file(self, data):
        detail = Path(data)
        user_check = User.objects.filter(username=detail.parent.name)
        if user_check.exists():
            file_data = Files.objects.create(
                        name=detail.name,
                        type=detail.name.split('.')[-1].lower(),
                        size=detail.stat().st_size,
                        owner=detail.parent.name
                    )
            bomiot_signals.send(sender=Files, msg={
                'models': 'Files',
                'type': 'create',
                'data': {
                    'id': file_data.id,
                    'name': file_data.name,
                    'type': file_data.type,
                    'size': file_data.size,
                    'owner': file_data.owner,
                    'shared_to': file_data.shared_to
                }
            })


    def modified_file(self, data):
        detail = Path(data)
        user_check = User.objects.filter(username=detail.parent.name)
        if user_check.exists():
            file_data_check = Files.objects.filter(name=detail.name, owner=detail.parent.name)
            if file_data_check.exists():
                old_instance = file_data_check.first()
                new_instance = Files.objects.filter(name=detail.name, owner=detail.parent.name).first()
                new_instance.size=detail.stat().st_size
                new_instance.is_delete = False
                new_instance.save()
                instance = Files.objects.filter(id=old_instance.id).first()
                data_before_update = model_to_dict(old_instance)
                data_after_update = model_to_dict(instance)
                data_before_update['created_time'] = old_instance.created_time
                data_after_update['created_time'] = instance.created_time
                data_before_update['updated_time'] = old_instance.updated_time
                data_after_update['updated_time'] = instance.updated_time
                updated_fields = compare_dicts(data_before_update, data_after_update)
                bomiot_signals.send(sender=Files, msg={
                    'models': 'Files',
                    'type': 'update',
                    'data': {
                        'id': instance.id,
                        'name': instance.name,
                        'type': instance.type,
                        'size': instance.size,
                        'owner': instance.owner,
                        'shared_to': instance.shared_to
                    },
                    'updated_fields': updated_fields
                })

    def deleted_file(self, data):
        detail = Path(data)
        user_check = User.objects.filter(username=detail.parent.name)
        if user_check.exists():
            Files.objects.filter(name=detail.name, owner=detail.parent.name).delete()

    def moved_file(self, data):
        move_from = Path(data[0])
        move_from_user = User.objects.filter(username=move_from.parent.name)
        if move_from_user.exists():
            Files.objects.filter(name=move_from.name, owner=move_from.parent.name).delete()
        move_to = Path(data[1])
        move_to_user = User.objects.filter(username=move_to.parent.name)
        if move_to_user.exists():
            Files.objects.create(
                name=move_to.name,
                type=move_to.name.split('.')[-1].lower(),
                size=move_to.stat().st_size,
                owner=move_to.parent.name,
                shared_to=''
            )

observer = Observer()
event_handler = MyHandler()


class ObserverManager(Thread):
    """
        Observer of media folder
    """

    def __init__(self, observer):
        """
        init manager
        :param observer:
        """
        super(ObserverManager, self).__init__()
        self.observer = observer
        self.observer.start()

    def run(self):
        """
        heart beat detect
        :return:
        """
        self.observer.schedule(event_handler, path=settings.MEDIA_ROOT, recursive=True)


# init observer manager
ob = ObserverManager(observer)



import orjson

from rest_framework.viewsets import ModelViewSet
from rest_framework.response import Response
from rest_framework import status
from django.db import transaction
from bomiot.server.core import models, serializers, filter
from bomiot.server.core.signal import bomiot_data_signals
from rest_framework.filters import OrderingFilter
from rest_framework.exceptions import MethodNotAllowed
from django_filters.rest_framework import DjangoFilterBackend
from bomiot.server.core.page import DataCorePageNumberPagination
from bomiot.server.core.utils import all_fields_empty, queryset_to_dict, compare_dicts


class CapitalList(ModelViewSet):
    """
        list:
            Response a Capital data list（all）
    """
    pagination_class = DataCorePageNumberPagination
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ["id", "created_time", "updated_time", ]
    filter_class = filter.CapitalFilter

    def get_queryset(self):
        query_params = self.request.query_params.get('params', {})
        if query_params:
            query_str = query_params.replace("'", '"')
            query_str = query_str.replace("true", "True").replace("false", "False")
            query_data = orjson.loads(query_str)
            if all_fields_empty(query_data):
                query_data = {}
        else:
            query_data = {}
        if "is_delete" not in query_data:
            query_data['is_delete'] = False
        ordering = query_data.pop('order_by', '-id')
        if not any(key.startswith('data__department') for key in query_data):
            if self.request.auth.department == 0:
                query_data['data__department__gte'] = 0
            else:
                query_data['data__department'] = self.request.auth.department
        return models.Capital.objects.filter(**query_data).order_by(ordering)

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.CapitalSerializer
        else:
            raise MethodNotAllowed(self.request.method)
        
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)
    

class CapitalCreate(ModelViewSet):
    """
    A powerful ViewSet for database management and feedback mechanism
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.CapitalFilter
    queryset = models.Capital.objects.filter(is_delete=False)

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.CapitalSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, *args, **kwargs):
        """
        Override the create method, combining send_robust and transaction management
        """
        data = self.request.data
        try:
            with transaction.atomic():
                responses = bomiot_data_signals.send_robust(sender=self.__class__,
                                                            request=self.request,
                                                            mode='create',
                                                            data=data)
                for receiver, response in responses:
                    if isinstance(response, Exception):
                        raise response
                    if isinstance(response, dict) and response.get("msg"):
                        data['department'] = self.request.auth.department if self.request.auth else 0
                        data['creater'] = self.request.auth.username
                        models.Capital.objects.create(data=data)
                        return Response(response)
                    if isinstance(response, dict) and response.get("detail"):
                        return Response(response)
                    if isinstance(response, dict) and response.get("login"):
                        return Response(response)
            return Response(data, status=status.HTTP_201_CREATED)
        except ValueError as e:
            with transaction.atomic():
                transaction.set_rollback(True)
                return Response({"detail": str(e)}, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            with transaction.atomic():
                transaction.set_rollback(True)
                return Response({"detail": f"An unexpected error occurred: {str(e)}"}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
            

class CapitalUpdate(ModelViewSet):
    """
    A powerful ViewSet for database management and feedback mechanism
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.CapitalFilter
    queryset = models.Capital.objects.filter(is_delete=False)

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.CapitalSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, *args, **kwargs):
        """
        Override the create method, combining send_robust and transaction management
        """
        data = self.request.data
        db_data = models.Capital.objects.filter(id=data.get('id'), is_delete=False)
        db_check_data = queryset_to_dict(db_data)
        updated_fields = compare_dicts(db_check_data[0], data)
        try:
            with transaction.atomic():
                responses = bomiot_data_signals.send_robust(sender=self.__class__,
                                                            request=self.request,
                                                            mode='update',
                                                            data=data,
                                                            updated_fields=updated_fields)
                for receiver, response in responses:
                    if isinstance(response, Exception):
                        raise response
                    if isinstance(response, dict) and response.get("msg"):
                        data.pop('id', None)
                        data.pop('is_delete', None)
                        data.pop('created_time', None)
                        data.pop('updated_time', None)
                        db_data.update(data=data)
                        return Response(response)
                    if isinstance(response, dict) and response.get("detail"):
                        return Response(response)
                    if isinstance(response, dict) and response.get("login"):
                        return Response(response)
            return Response(data, status=status.HTTP_201_CREATED)
        except ValueError as e:
            with transaction.atomic():
                transaction.set_rollback(True)
                return Response({"detail": str(e)}, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            with transaction.atomic():
                transaction.set_rollback(True)
                return Response({"detail": f"An unexpected error occurred: {str(e)}"}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class CapitalDelete(ModelViewSet):
    """
    A powerful ViewSet for database management and feedback mechanism
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = filter.CapitalFilter
    queryset = models.Capital.objects.filter(is_delete=False)

    def get_serializer_class(self):
        if self.action in ['create']:
            return serializers.CapitalSerializer
        else:
            raise MethodNotAllowed(self.request.method)

    def create(self, request, *args, **kwargs):
        """
        Override the create method, combining send_robust and transaction management
        """
        data = self.request.data
        db_data = models.Capital.objects.filter(id=data.get('id'), is_delete=False)
        try:
            with transaction.atomic():
                responses = bomiot_data_signals.send_robust(sender=self.__class__,
                                                            request=self.request,
                                                            mode='delete',
                                                            data=data)
                for receiver, response in responses:
                    if isinstance(response, Exception):
                        raise response
                    if isinstance(response, dict) and response.get("msg"):
                        db_data.update(is_delete=True)
                        return Response(response)
                    if isinstance(response, dict) and response.get("detail"):
                        return Response(response)
                    if isinstance(response, dict) and response.get("login"):
                        return Response(response)
            return Response(data, status=status.HTTP_201_CREATED)
        except ValueError as e:
            with transaction.atomic():
                transaction.set_rollback(True)
                return Response({"detail": str(e)}, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            with transaction.atomic():
                transaction.set_rollback(True)
                return Response({"detail": f"An unexpected error occurred: {str(e)}"}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


import orjson

from os.path import join
from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _
import bomiot_data_control
from bomiot.server.core.signal import bomiot_data_signals
import bomiot_control
from bomiot.server.core.signal import bomiot_signals

# get user_model
User = get_user_model()


@admin.register(User)
class UserAdmin(BaseUserAdmin):
    """
    user admin
    """
    # difine the form for creating and updating user
    fieldsets = (
        (None, {"fields": ("username", "password")}),
        (_("Personal info"), {"fields": ("phone", "email", "type")}),
        (
            _("Permissions"),
            {
                "fields": (
                    "is_active",
                    "is_staff",
                    "is_superuser",
                ),
            },
        ),
        (_("Important dates"), {"fields": ("last_login", "date_joined")}),
    )

    # define the form for creating and updating user
    list_display = (
        "username",
        "phone",
        "email",
        "type",
        "is_staff",
        "is_superuser",
        "is_active",
    )

    # add the filter for the list view
    list_filter = ("is_staff", "is_superuser", "is_active", "type")

    # add the search fields for the list view
    search_fields = ("username", "email", "phone")

bomiot_data_signals.connect(bomiot_data_control.data_callback, weak=False)
bomiot_signals.connect(bomiot_control.server_callback, weak=False)




def url_ignore():
    """
    return a list of urls that should be ignored by the middleware
    """
    admin_urls = [
        '/admin/', '/admin/login/', '/admin/logout/', '/admin/password_change/', '/admin/password_change/done/',
        '/admin/autocomplete/', '/admin/jsi18n/', '/admin/r/<int:content_type_id>/<path:object_id>/',
        '/admin/auth/group/', '/admin/auth/group/add/', '/admin/auth/group/<path:object_id>/history/',
        '/admin/auth/group/<path:object_id>/delete/', '/admin/auth/group/<path:object_id>/change/',
        '/admin/auth/group/<path:object_id>/', '/admin/django_apscheduler/djangojob/',
        '/admin/django_apscheduler/djangojob/add/', '/admin/django_apscheduler/djangojob/<path:object_id>/history/',
        '/admin/django_apscheduler/djangojob/<path:object_id>/delete/', '/admin/django_apscheduler/djangojob/<path:object_id>/change/',
        '/admin/django_apscheduler/djangojob/<path:object_id>/', '/admin/django_apscheduler/djangojobexecution/',
        '/admin/django_apscheduler/djangojobexecution/add/', '/admin/django_apscheduler/djangojobexecution/<path:object_id>/history/',
        '/admin/django_apscheduler/djangojobexecution/<path:object_id>/delete/', '/admin/django_apscheduler/djangojobexecution/<path:object_id>/change/',
        '/admin/django_apscheduler/djangojobexecution/<path:object_id>/', '/admin/core/user/<id>/password/',
        '/admin/core/user/', '/admin/core/user/add/', '/admin/core/user/<path:object_id>/history/',
        '/admin/core/user/<path:object_id>/delete/', '/admin/core/user/<path:object_id>/change/',
        '/admin/core/user/<path:object_id>/', '/admin/(?P<app_label>auth|django_apscheduler|core)/', '/admin/(?P<url>.*)'
    ]

    public_urls = [
        '/', '/login/', '/logout/', '/register/', '/checktoken/', '/favicon.ico'
    ]

    static_urls = [
        '/css/.*', '/js/.*', '/assets/.*', '/statics/.*', '/fonts/.*', '/icons/.*',
        '/static/(?P<path>.*)', '/media/(?P<path>.*)'
    ]

    # Add any other URLs that should be ignored by the middleware
    return admin_urls + public_urls + static_urls


import inspect
import importlib
import json
import bomiot_token

from os.path import join
from datetime import datetime, date
from configparser import ConfigParser
from django.dispatch import receiver
from django.db import transaction
from django.conf import settings
from bomiot.cmd import create_key
from bomiot.server.core.signal import bomiot_signals, bomiot_data_signals
from bomiot.server.core.utils import receiver_callback, receiver_file_callback, receiver_server_callback
from bomiot.server.core.utils import sync_write_file
from bomiot.server.core.models import API
from bomiot.server.core.models import JobList


def calculate_days_difference(expire_timestamp):
    try:
        expire_date = datetime.fromtimestamp(expire_timestamp).date()
        today = date.today()
        days_difference = (expire_date - today).days
        return days_difference
    except Exception as e:
        return None

def init():
    """
    Initialize the module by importing necessary modules and setting up signal receivers.
    """
    key = settings.KEY
    if (key == ''):
        key = create_key.auth_key_force()
    mac, expire_time = bomiot_token.verify_info(key)
    days_diff = calculate_days_difference(int(expire_time))
    if days_diff is not None:
        if days_diff <= 0:
            edit_setup = False
            CONFIG = ConfigParser()
            setup_ini_path = join(settings.WORKING_SPACE, 'setup.ini')
            CONFIG.read(setup_ini_path, encoding='utf-8')
            if CONFIG.get('database', 'name') != "sqlite":
                CONFIG.set('database', 'name', 'sqlite')
                edit_setup = True
            if CONFIG.getint('file', 'file_size') != 102400:
                CONFIG.set('file', 'file_size', str(102400))
                edit_setup = True
            if CONFIG.get('file', 'file_extension') != 'py,png,jpg,jpeg,gif,bmp,webp,txt,md,html,htm,js,css,json,xml,csv,xlsx,xls,ppt,pptx,doc,docx,pdf':
                CONFIG.set('file', 'file_extension', 'py,png,jpg,jpeg,gif,bmp,webp,txt,md,html,htm,js,css,json,xml,csv,xlsx,xls,ppt,pptx,doc,docx,pdf')
                edit_setup = True
            if edit_setup is True:
                CONFIG.write(open(join(settings.WORKING_SPACE, 'setup.ini'), "wt"))
            # core_watch.start()
        elif days_diff > 0 and days_diff < 30:
            print(f"Keys will expires in {days_diff} days")

@receiver(bomiot_data_signals)
def data_callback(**kwargs):
    """
    Signal receiver to handle the received signal
    """
    path = kwargs.get('request').path
    api_obj = API.objects.filter(api=path).first()
    if api_obj is None:
        return None
    value = api_obj.func_name
    feedback = receiver_callback(kwargs, value)
    return feedback

@receiver(bomiot_signals)
def server_callback(sender, **kwargs):
    """
    Signal receiver to handle server signal
    """
    data = kwargs.get('msg')
    if data.get('models') == 'Pids':
        receiver_server_callback(data.get('data'), 'pid_get')
    elif data.get('models') == 'Network':
        receiver_server_callback(data.get('data'), 'network_get')
    elif data.get('models') == 'Disk':
        receiver_server_callback(data.get('data'), 'disk_get')
    elif data.get('models') == 'Memory':
        receiver_server_callback(data.get('data'), 'memory_get')
    elif data.get('models') == 'CPU':
        receiver_server_callback(data.get('data'), 'cpu_get')
    elif data.get('models') == 'Files':
        receiver_file_callback(data.get('data'), 'file_get')
    elif data.get('models', '') == 'Function':
        if sender == sync_write_file:
            sync_write_file(
                kwargs.get('file_path'),
                kwargs.get('file_data')
            )
        else:
            job_func = importlib.import_module(f'{sender.__module__}')
            job_function = getattr(job_func, sender.__name__)
            sig = inspect.signature(job_function)
            if sig.parameters:
                func_kwargs = {k: v for k, v in kwargs.items() if k in sig.parameters}
                job_function(**func_kwargs)
            else:
                job_function()
    elif data.get('models', '') == 'JobList':
        job_data = data.get('data', {})
        job_id = f"{inspect.getmodule(sender).__name__}-{sender.__name__}-{str(job_data)}"
        with transaction.atomic():
            job, created = JobList.objects.get_or_create(
                job_id=job_id,
                defaults={
                    'module_name': inspect.getmodule(sender).__name__,
                    'func_name': sender.__name__,
                    'trigger': job_data.get('trigger'),
                    'description': job_data.get('description', ''),
                    'configuration': json.dumps(job_data)
                }
            )
            if not created:
                job.trigger = job_data.get('trigger')
                job.description = job_data.get('description', '')
                job.configuration = json.dumps(job_data)
                job.save()
    else:
        return
    return


import psutil

from rest_framework import viewsets

from bomiot.server.core import serializers, models, filter
from bomiot.server.core.page import CorePageNumberPagination
from bomiot.server.core.permission import NormalPermission
from rest_framework.filters import OrderingFilter
from rest_framework.exceptions import MethodNotAllowed
from django_filters.rest_framework import DjangoFilterBackend
from bomiot_message import others_message_return

from django.db.models import Sum


class PIDList(viewsets.ModelViewSet):
    """
        list:
            Response a PID data list（all）
    """
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ["id", "created_time", "updated_time", ]
    filter_class = filter.PidsFilter

    def get_queryset(self):
        if self.request.user:
            query_data = {}
            query_data['is_delete'] = False
            query_data['name__icontains'] = self.request.query_params.get('search')
            return models.Pids.objects.filter(**query_data).order_by('-memory_usage', '-cpu_usage')
        else:
            return models.Pids.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.PidsSerializer
        else:
            raise MethodNotAllowed(self.request.method)
        
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)
    

class CPUList(viewsets.ModelViewSet):
    """
        list:
            Response a CPU data list（all）
    """
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ["id", "created_time", "updated_time", ]
    filter_class = filter.CPUFilter

    def get_queryset(self):
        if self.request.user:
            query_data = {}
            query_data['is_delete'] = False
            return models.CPU.objects.filter(**query_data).order_by('-id')
        else:
            return models.CPU.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.CPUSerializer
        else:
            raise MethodNotAllowed(self.request.method)
        
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)
    

class MemoryList(viewsets.ModelViewSet):
    """
        list:
            Response a Memory data list（all）
    """
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ["id", "created_time", "updated_time", ]
    filter_class = filter.MemoryFilter

    def get_queryset(self):
        if self.request.user:
            query_data = {}
            query_data['is_delete'] = False
            return models.Memory.objects.filter(**query_data).order_by('-id')
        else:
            return models.Memory.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.MemorySerializer
        else:
            raise MethodNotAllowed(self.request.method)
        
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)
    

class DiskList(viewsets.ModelViewSet):
    """
        list:
            Response a Disk data list（all）
    """
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ["id", "created_time", "updated_time", ]
    filter_class = filter.DiskFilter

    def get_queryset(self):
        if self.request.user:
            query_data = {}
            query_data['is_delete'] = False
            return models.Disk.objects.filter(**query_data).order_by('-id')
        else:
            return models.Disk.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.DiskSerializer
        else:
            raise MethodNotAllowed(self.request.method)
        
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)
    

class NetworkList(viewsets.ModelViewSet):
    """
        list:
            Response a Network data list（all）
    """
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ["id", "created_time", "updated_time", ]
    filter_class = filter.NetworkFilter

    def get_queryset(self):
        if self.request.user:
            query_data = {}
            query_data['is_delete'] = False
            return models.Network.objects.filter(**query_data).order_by('-id')
        else:
            return models.Network.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.NetworkSerializer
        else:
            raise MethodNotAllowed(self.request.method)
        
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)
    

class ServerChartsPage(CorePageNumberPagination):
    def __init__(self, *args, **kwargs):
        self.cpu_data = {}
        self.cpu_data['xAxis_list'] = []
        self.cpu_usage_list = []

        self.memory_data = {}
        self.memory_data['xAxis_list'] = []
        self.memory_used_list = []
        self.memory_free_list = []

        self.disk_data = {}
        self.disk_data['xAxis_list'] = []
        self.disk_used_list = []
        self.disk_free_list = []

        self.network_data = {}
        self.network_data['xAxis_list'] = []
        self.bytes_sent_list = []
        self.bytes_recv_list = []
        super(ServerChartsPage, self).__init__(*args, **kwargs)


    def get_cpu_return_data(self, data) -> dict:
        self.cpu_data['xAxis_list'].append(data.created_time.strftime("%H:%M"))
        self.cpu_usage_list.append(data.cpu_usage)
        return data
    
    def get_memory_return_data(self, data) -> dict:
        self.memory_data['xAxis_list'].append(data.created_time.strftime("%H:%M"))
        self.memory_used_list.append(data.used)
        self.memory_free_list.append(data.free)
        return data

    def get_network_return_data(self, data) -> dict:
        self.network_data['xAxis_list'].append(data.created_time.strftime("%H:%M"))
        self.bytes_sent_list.append(data.bytes_sent)
        self.bytes_recv_list.append(data.bytes_recv)
        return data
    
    def query_data_add(self) -> list:
        cpu_list = models.CPU.objects.filter().order_by('-id')[:30]
        cpu_data_list = list(map(lambda data: self.get_cpu_return_data(data), cpu_list))
        
        memory_list = models.Memory.objects.filter().order_by('-id')[:30]
        memory_data_list = list(map(lambda data: self.get_memory_return_data(data), memory_list))

        partitions = psutil.disk_partitions()
        for i in range(len(partitions)):
            try:
                disk_usage = psutil.disk_usage(partitions[i].mountpoint)
                self.disk_data['xAxis_list'].append(partitions[i].mountpoint)
                self.disk_used_list.append(float(disk_usage.used))
                self.disk_free_list.append(float(disk_usage.free))
            except PermissionError:
                print(f"{partitions[i].mountpoint}")

        network_list = models.Network.objects.filter().order_by('-id')[:30]
        network_data_list = list(map(lambda data: self.get_network_return_data(data), network_list))
        
        self.cpu_data['xAxis_list'] = list(reversed(self.cpu_data['xAxis_list']))
        self.cpu_data['title'] = others_message_return(self.request.META.get('HTTP_LANGUAGE', ''), "CPU Info")
        self.cpu_data['series_list'] = [
            { 'name': others_message_return(self.request.META.get('HTTP_LANGUAGE', ''), "CPU Usage"), 'data': list(reversed(self.cpu_usage_list)) }
        ]

        self.memory_data['xAxis_list'] = list(reversed(self.memory_data['xAxis_list']))
        self.memory_data['title'] = others_message_return(self.request.META.get('HTTP_LANGUAGE', ''), "Memory Info")
        self.memory_data['series_list'] = [
            { 'name': others_message_return(self.request.META.get('HTTP_LANGUAGE', ''), "Memory Usad"), 'data': list(reversed(self.memory_used_list)) },
            { 'name': others_message_return(self.request.META.get('HTTP_LANGUAGE', ''), "Memory Free"), 'data': list(reversed(self.memory_free_list)) },
        ]

        self.disk_data['title'] = others_message_return(self.request.META.get('HTTP_LANGUAGE', ''), "Disk Info")
        self.disk_data['series_list'] = [
            { 'name': others_message_return(self.request.META.get('HTTP_LANGUAGE', ''), "Disk Used"), 'data': self.disk_used_list },
            { 'name': others_message_return(self.request.META.get('HTTP_LANGUAGE', ''), "Disk Free"), 'data': self.disk_free_list },
        ]

        self.network_data['xAxis_list'] = list(reversed(self.network_data['xAxis_list']))
        self.network_data['title'] = others_message_return(self.request.META.get('HTTP_LANGUAGE', ''), "Network Info")
        self.network_data['series_list'] = [
            { 'name': others_message_return(self.request.META.get('HTTP_LANGUAGE', ''), "Bytes Sent"), 'data': list(reversed(self.bytes_sent_list)) },
            { 'name': others_message_return(self.request.META.get('HTTP_LANGUAGE', ''), "Bytes Received"), 'data': list(reversed(self.bytes_recv_list)) },
        ]
        return [
            ("cpu_data", self.cpu_data),
            ("memory_data", self.memory_data),
            ("disk_data", self.disk_data),
            ("network_data", self.network_data)
        ]

class ServerCharts(viewsets.ModelViewSet):
    """
        list:
            Response a Server Charts data list（all）
    """
    pagination_class = ServerChartsPage
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ["id", "created_time", "updated_time", ]
    filter_class = filter.NetworkFilter

    def get_queryset(self):
        return models.Network.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.NetworkSerializer
        else:
            raise MethodNotAllowed(self.request.method)
        
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)
    
class PIDChartsPage(CorePageNumberPagination):
    def __init__(self, *args, **kwargs):
        self.pid_data = {}
        self.pid_data['title'] = 'PID Tree Map'
        self.pid_series_list = []
        super(PIDChartsPage, self).__init__(*args, **kwargs)
    
    def get_pid_return_data(self, data) -> dict:
        pid_append_data = {}
        pid_append_data['name'] = data.get('name')
        pid_append_data['value'] = data.get('total_memory')
        self.pid_series_list.append(pid_append_data)
        return data

    def query_data_add(self) -> list:
        pid_list = models.Pids.objects.values('name').annotate(total_memory=Sum('memory')).order_by('-total_memory')
        pid_data_list = list(map(lambda data: self.get_pid_return_data(data), pid_list))
        self.pid_data['series_list'] = self.pid_series_list

        return [
            ("pid_data", self.pid_data),
        ]

class PIDCharts(viewsets.ModelViewSet):
    """
        list:
            Response a PID Charts data list（all）
    """
    pagination_class = PIDChartsPage
    permission_classes = [NormalPermission, ]
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ["id", "created_time", "updated_time", ]
    filter_class = filter.PidsFilter

    def get_queryset(self):
        return models.Pids.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return serializers.PidsSerializer
        else:
            raise MethodNotAllowed(self.request.method)
        
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)


from django.db import models

class test(models.Model):
    is_delete = models.BooleanField(default=False, verbose_name='Delete Label')
    created_time = models.DateTimeField(auto_now_add=True, verbose_name="Created Time")
    updated_time = models.DateTimeField(auto_now=True, blank=True, null=True, verbose_name="Updated Time")

    class Meta:
        ordering = ['-id']


class xxxx(models.Model):
    is_delete = models.BooleanField(default=False, verbose_name='Delete Label')
    created_time = models.DateTimeField(auto_now_add=True, verbose_name="Created Time")
    updated_time = models.DateTimeField(auto_now=True, blank=True, null=True, verbose_name="Updated Time")

    class Meta:
        ordering = ['-id']


from rest_framework import serializers
from django.contrib.auth import get_user_model
from bomiot.server.core import models

User = get_user_model()

class UserSerializer(serializers.ModelSerializer):
    username = serializers.CharField(read_only=True, required=False)
    email = serializers.CharField(read_only=True, required=False)
    phone = serializers.CharField(read_only=True, required=False)
    permission = serializers.JSONField(read_only=True, required=False)
    request_limit = serializers.IntegerField(read_only=True, required=False)
    team = serializers.IntegerField(read_only=True, required=False)
    department = serializers.IntegerField(read_only=True, required=False)
    is_active = serializers.BooleanField(read_only=True, required=False)
    date_joined = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    last_login = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')

    class Meta:
        model = User
        fields = ['id', 'username', 'email', 'phone', 'is_active', 'request_limit', 'team', 'department', 'permission', 'date_joined', 'last_login', 'updated_time']
        read_only_fields = ['id']


class PermissionSerializer(serializers.ModelSerializer):
    """
    Permission Serializer
    """
    api = serializers.CharField(read_only=True, required=False)
    name = serializers.CharField(read_only=True, required=False)

    class Meta:
        model = models.Permission
        fields = ['api', 'name']
        read_only_fields = ['api']


class TeamSerializer(serializers.ModelSerializer):
    """
    Team Serializer
    """
    name = serializers.CharField(read_only=True, required=False)
    permission = serializers.JSONField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')

    class Meta:
        model = models.Team
        fields = ['id', 'name', 'permission', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class DepartmentSerializer(serializers.ModelSerializer):
    """
    Department Serializer
    """
    name = serializers.CharField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')

    class Meta:
        model = models.Team
        fields = ['id', 'name', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class FileSerializer(serializers.ModelSerializer):
    """
    File Serializer
    """
    name = serializers.CharField(read_only=True, required=False)
    type = serializers.CharField(read_only=True, required=False)
    size = serializers.IntegerField(read_only=True, required=False)
    owner = serializers.CharField(read_only=True, required=False)
    shared_to = serializers.CharField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')

    class Meta:
        model = models.Files
        fields = ['id', 'name', 'type', 'size', 'owner', 'shared_to', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class APISerializer(serializers.ModelSerializer):
    """
    API Serializer
    """
    method = serializers.CharField(read_only=True, required=False)
    api = serializers.CharField(read_only=True, required=False)
    func_name = serializers.CharField(read_only=True, required=False)
    name = serializers.CharField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    
    class Meta:
        model = models.Example
        fields = ['id', 'method', 'api', 'func_name', 'name', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class ExampleSerializer(serializers.ModelSerializer):
    """
    Example Serializer
    """
    data = serializers.JSONField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    
    class Meta:
        model = models.Example
        fields = ['id', 'data', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class PidsSerializer(serializers.ModelSerializer):
    """
    Pid Serializer
    """
    pid = serializers.IntegerField(read_only=True, required=False)
    name = serializers.CharField(read_only=True, required=False)
    memory = serializers.IntegerField(read_only=True, required=False)
    memory_usage = serializers.FloatField(read_only=True, required=False)
    cpu_usage = serializers.FloatField(read_only=True, required=False)
    create_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')

    class Meta:
        model = models.Pids
        fields = ['pid', 'name', 'memory', 'create_time', 'memory_usage', 'cpu_usage', 'created_time', 'updated_time']
        read_only_fields = ['pid']


class PyPiSerializer(serializers.ModelSerializer):
    """
    PyPi Serializer
    """
    category = serializers.CharField(read_only=True, required=False)
    date = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d')
    percent = serializers.FloatField(read_only=True, required=False)
    downloads = serializers.IntegerField(read_only=True, required=False)

    class Meta:
        model = models.PyPi
        fields = ['id', 'category', 'date', 'percent', 'downloads']
        read_only_fields = ['id']


class CPUSerializer(serializers.ModelSerializer):
    """
    CPU Serializer
    """
    cpu_usage = serializers.FloatField(read_only=True, required=False)
    physical_cores = serializers.IntegerField(read_only=True, required=False)
    logical_cores = serializers.IntegerField(read_only=True, required=False)
    cpu_frequency = serializers.CharField(read_only=True, required=False)
    min_cpu_frequency = serializers.CharField(read_only=True, required=False)
    max_cpu_frequency = serializers.CharField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')

    class Meta:
        model = models.CPU
        fields = ['id', 'cpu_usage', 'physical_cores', 'logical_cores', 'cpu_frequency', 'min_cpu_frequency', 'max_cpu_frequency', 'created_time', 'updated_time']
        read_only_fields = ['id']


class MemorySerializer(serializers.ModelSerializer):
    """
    Memory Serializer
    """
    total = serializers.IntegerField(read_only=True, required=False)
    used = serializers.IntegerField(read_only=True, required=False)
    free = serializers.IntegerField(read_only=True, required=False)
    percent = serializers.FloatField(read_only=True, required=False)
    swap_total = serializers.IntegerField(read_only=True, required=False)
    swap_used = serializers.IntegerField(read_only=True, required=False)
    swap_free = serializers.IntegerField(read_only=True, required=False)
    swap_percent = serializers.FloatField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')

    class Meta:
        model = models.Memory
        fields = ['id', 'total', 'used', 'free', 'percent', 'swap_total', 'swap_used', 'swap_free', 'swap_percent', 'created_time', 'updated_time']
        read_only_fields = ['id']


class DiskSerializer(serializers.ModelSerializer):
    """
    Disk Serializer
    """
    device = serializers.CharField(read_only=True, required=False)
    mountpoint = serializers.CharField(read_only=True, required=False)
    total = serializers.IntegerField(read_only=True, required=False)
    used = serializers.IntegerField(read_only=True, required=False)
    free = serializers.IntegerField(read_only=True, required=False)
    percent = serializers.FloatField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')

    class Meta:
        model = models.Disk
        fields = ['id', 'device', 'mountpoint', 'total', 'used', 'free', 'percent', 'created_time', 'updated_time']
        read_only_fields = ['id']


class NetworkSerializer(serializers.ModelSerializer):
    """
    Network Serializer
    """
    bytes_sent = serializers.IntegerField(read_only=True, required=False)
    bytes_recv = serializers.IntegerField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')

    class Meta:
        model = models.Network
        fields = ['id', 'bytes_sent', 'bytes_recv', 'created_time', 'updated_time']
        read_only_fields = ['id']


class GoodsSerializer(serializers.ModelSerializer):
    """
    Goods Serializer
    """
    data = serializers.JSONField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    
    class Meta:
        model = models.Goods
        fields = ['id', 'data', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class BinSerializer(serializers.ModelSerializer):
    """
    Bin Serializer
    """
    data = serializers.JSONField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    
    class Meta:
        model = models.Bin
        fields = ['id', 'data', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class StockSerializer(serializers.ModelSerializer):
    """
    Stock Serializer
    """
    data = serializers.JSONField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    
    class Meta:
        model = models.Stock
        fields = ['id', 'data', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class CapitalSerializer(serializers.ModelSerializer):
    """
    Capital Serializer
    """
    data = serializers.JSONField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    
    class Meta:
        model = models.Capital
        fields = ['id', 'data', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class SupplierSerializer(serializers.ModelSerializer):
    """
    Supplier Serializer
    """
    data = serializers.JSONField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    
    class Meta:
        model = models.Supplier
        fields = ['id', 'data', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class CustomerSerializer(serializers.ModelSerializer):
    """
    Customer Serializer
    """
    data = serializers.JSONField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    
    class Meta:
        model = models.Customer
        fields = ['id', 'data', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class ASNSerializer(serializers.ModelSerializer):
    """
    ASN Serializer
    """
    data = serializers.JSONField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    
    class Meta:
        model = models.ASN
        fields = ['id', 'data', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class DNSerializer(serializers.ModelSerializer):
    """
    DN Serializer
    """
    data = serializers.JSONField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    
    class Meta:
        model = models.DN
        fields = ['id', 'data', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class PurchaseSerializer(serializers.ModelSerializer):
    """
    Purchase Serializer
    """
    data = serializers.JSONField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    
    class Meta:
        model = models.Purchase
        fields = ['id', 'data', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class BarSerializer(serializers.ModelSerializer):
    """
    Bar Serializer
    """
    data = serializers.JSONField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    
    class Meta:
        model = models.Bar
        fields = ['id', 'data', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class FeeSerializer(serializers.ModelSerializer):
    """
    Fee Serializer
    """
    data = serializers.JSONField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    
    class Meta:
        model = models.Fee
        fields = ['id', 'data', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


class DriverSerializer(serializers.ModelSerializer):
    """
    Fee Serializer
    """
    data = serializers.JSONField(read_only=True, required=False)
    is_delete = serializers.BooleanField(read_only=True, required=False)
    created_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    updated_time = serializers.DateTimeField(read_only=True, required=False, format='%Y-%m-%d %H:%M:%S')
    
    class Meta:
        model = models.Fee
        fields = ['id', 'data', 'is_delete', 'created_time', 'updated_time']
        read_only_fields = ['id']


from django.contrib.auth import get_user_model
from bomiot.server.core.utils import contains_value


User = get_user_model()


class BasePermission:
    """
    Base permission class
    """
    def has_permission(self, request, view) -> bool:
        raise NotImplementedError("need has_permission function")


class CorePermission(BasePermission):
    """
    core permission class, check if user has permission to access the path
    """
    def has_permission(self, request, view) -> bool:
        if request.user:
            return contains_value(request.auth.permission, request.path)
        return False


class NormalPermission(BasePermission):
    """
    normal permission class, allow all users to access the path
    """
    def has_permission(self, request, view) -> bool:
        return True